@@ -489,7 +489,9 @@ namespace System.Data {
 			DataColumn column = _table.Columns[columnName];
 			_table.ChangingDataColumn (this, column, val);
 				
-			if (_original < 0) { 
+                        if (_original < 0 || _original == _current) { 
+				// really add a row cache, if _original is not there & 
+				// make row modified
 				_original = Table.RecordCache.NewRecord();
 			}
 			CheckValue (val, column);
@@ -1036,7 +1038,9 @@ namespace System.Data {
 			int numColumn = parentColumns.Length;
 			if (HasVersion(version)) {
 				Index indx = relation.ParentTable.GetIndexByColumns (parentColumns);
-				if (indx != null) { // get the child rows from the index
+				if (indx != null &&
+				    (Table == null || Table.DataSet == null ||   
+				   Table.DataSet.EnforceConstraints)) { // get the child rows from the index
 					Node[] childNodes = indx.FindAllSimple(childColumns, IndexFromVersion(version));
 					for (int i = 0; i < childNodes.Length; i++) {
 						rows.Add (childNodes[i].Row);
