@@ -330,13 +330,19 @@ namespace System.Data {
 				tempTable.AcceptChanges ();
 		}
 
+                /// <summary>
+                /// Clears all the tables
+                /// </summary>
 		public void Clear ()
 		{
 			if (_xmlDataDocument != null)
 				throw new NotSupportedException ("Clear function on dataset and datatable is not supported when XmlDataDocument is bound to the DataSet.");
-			for (int t = 0; t < tableCollection.Count; t++) {
-				tableCollection[t].Clear ();
+                        bool enforceConstraints = this.EnforceConstraints;
+                        this.EnforceConstraints = false;
+                        for (int t = 0; t < tableCollection.Count; t++) {
+				tableCollection [t].Clear ();
 			}
+                        this.EnforceConstraints = enforceConstraints;
 		}
 
 		public virtual DataSet Clone ()
@@ -1188,16 +1194,22 @@ namespace System.Data {
 					case 0:
 						break;
 					case 1:
+						if (!oneRel.Nested)
+							break;
 						if (row.GetParentRow (oneRel) != null)
 							continue;
 						break;
 					case 2:
 						bool skip = false;
-						for (int i = 0; i < table.ParentRelations.Count; i++)
-							if (row.GetParentRow (table.ParentRelations [i]) != null) {
+						for (int i = 0; i < table.ParentRelations.Count; i++) {
+							DataRelation prel = table.ParentRelations [i];
+							if (!prel.Nested)
+								continue;
+							if (row.GetParentRow (prel) != null) {
 								skip = true;
 								continue;
 							}
+						}
 						if (skip)
 							continue;
 						break;
@@ -1270,7 +1282,8 @@ namespace System.Data {
 
 		private void WriteColumnAsAttribute (XmlWriter writer, XmlWriteMode mode, DataColumn col, DataRow row, DataRowVersion version)
 		{
-			WriteAttributeString (writer, mode, col.Namespace, col.Prefix, XmlConvert.EncodeLocalName (col.ColumnName), WriteObjectXml (row[col, version]));
+			if (!row.IsNull (col))
+				WriteAttributeString (writer, mode, col.Namespace, col.Prefix, XmlConvert.EncodeLocalName (col.ColumnName), WriteObjectXml (row[col, version]));
 		}
 
 		private void WriteTableElement (XmlWriter writer, XmlWriteMode mode, DataTable table, DataRow row, DataRowVersion version)
@@ -1384,12 +1397,17 @@ namespace System.Data {
 			XmlAttribute attr = null;
 			ArrayList atts = new ArrayList ();
 
+			attr = doc.CreateAttribute ("", "xmlns", xmlNSURI);
+			atts.Add (attr);
+
 			nsmgr.AddNamespace ("xs", XmlSchema.Namespace);
 			nsmgr.AddNamespace (XmlConstants.MsdataPrefix, XmlConstants.MsdataNamespace);
 			if (Namespace != "") {
 				nsmgr.AddNamespace (XmlConstants.TnsPrefix, Namespace);
 				nsmgr.AddNamespace (String.Empty, Namespace);
 			}
+			if (CheckExtendedPropertyExists ())
+				nsmgr.AddNamespace (XmlConstants.MspropPrefix, XmlConstants.MspropNamespace);
 
 			if (atts.Count > 0)
 				schema.UnhandledAttributes = atts.ToArray (typeof (XmlAttribute)) as XmlAttribute [];
@@ -1416,6 +1434,8 @@ namespace System.Data {
 
 			elem.UnhandledAttributes = atts.ToArray (typeof (XmlAttribute)) as XmlAttribute [];
 
+			AddExtendedPropertyAttributes (elem, ExtendedProperties, doc);
+
 			XmlSchemaComplexType complex = new XmlSchemaComplexType ();
 			elem.SchemaType = complex;
 
@@ -1446,7 +1466,7 @@ namespace System.Data {
 
 			schema.Items.Add (elem);
 			
-			AddConstraintsToSchema (elem, constraintPrefix, tables, relations);
+			AddConstraintsToSchema (elem, constraintPrefix, tables, relations, doc);
 			foreach (string prefix in nsmgr) {
 				string ns = nsmgr.LookupNamespace (nsmgr.NameTable.Get (prefix));
 				if (prefix != "xmlns" && prefix != "xml" && ns != null && ns != String.Empty)
@@ -1455,29 +1475,48 @@ namespace System.Data {
 			return schema;
 		}
 		
+		private bool CheckExtendedPropertyExists ()
+		{
+			if (ExtendedProperties.Count > 0)
+				return true;
+			foreach (DataTable dt in Tables) {
+				if (dt.ExtendedProperties.Count > 0)
+					return true;
+				foreach (DataColumn col in dt.Columns)
+					if (col.ExtendedProperties.Count > 0)
+						return true;
+				foreach (Constraint c in dt.Constraints)
+					if (c.ExtendedProperties.Count > 0)
+						return true;
+			}
+			foreach (DataRelation rel in Relations)
+				if (rel.ExtendedProperties.Count > 0)
+					return true;
+			return false;
+		}
+
 		// Add all constraints in all tables to the schema.
-		private void AddConstraintsToSchema (XmlSchemaElement elem, string constraintPrefix, DataTableCollection tables, DataRelationCollection relations) 
+		private void AddConstraintsToSchema (XmlSchemaElement elem, string constraintPrefix, DataTableCollection tables, DataRelationCollection relations, XmlDocument doc)
 		{
 			// first add all unique constraints.
-			Hashtable uniqueNames = AddUniqueConstraints (elem, constraintPrefix, tables);
+			Hashtable uniqueNames = AddUniqueConstraints (elem, constraintPrefix, tables, doc);
 			// Add all foriegn key constraints.
-			AddForeignKeys (uniqueNames, elem, constraintPrefix, relations);
+			AddForeignKeys (uniqueNames, elem, constraintPrefix, relations, doc);
 		}
 		
 		// Add unique constaraints to the schema.
 		// return hashtable with the names of all XmlSchemaUnique elements we created.
-		private Hashtable AddUniqueConstraints (XmlSchemaElement elem, string constraintPrefix, DataTableCollection tables)
+		private Hashtable AddUniqueConstraints (XmlSchemaElement elem, string constraintPrefix, DataTableCollection tables, XmlDocument doc)
 		{
-			XmlDocument doc = new XmlDocument();
 			Hashtable uniqueNames = new Hashtable();
 			foreach (DataTable table in tables) {
 				
-				foreach (Constraint constaint in table.Constraints) {
+				foreach (Constraint constraint in table.Constraints) {
 					
-					if (constaint is UniqueConstraint) {
+					if (constraint is UniqueConstraint) {
 						ArrayList attrs = new ArrayList ();
 						XmlAttribute attrib;
-						UniqueConstraint uqConst = (UniqueConstraint)constaint;
+						UniqueConstraint uqConst = (UniqueConstraint) constraint;
 						XmlSchemaUnique uniq = new XmlSchemaUnique ();
 						
 						// if column of the constraint is hidden do not write the constraint.
@@ -1516,10 +1555,13 @@ namespace System.Data {
 						XmlSchemaXPath field;
 						foreach (DataColumn column in uqConst.Columns) {
 				 			field = new XmlSchemaXPath();
-							field.XPath = constraintPrefix+column.ColumnName;
+							string typePrefix = column.ColumnMapping == MappingType.Attribute ? "@" : "";
+							field.XPath = typePrefix + constraintPrefix + column.ColumnName;
 							uniq.Fields.Add(field);
 						}
 				
+						AddExtendedPropertyAttributes (uniq, constraint.ExtendedProperties, doc);
+
 						elem.Constraints.Add (uniq);
 						uniqueNames.Add (uniq.Name, null);
 					}
@@ -1529,11 +1571,10 @@ namespace System.Data {
 		}
 		
 		// Add the foriegn keys to the schema.
-		private void AddForeignKeys (Hashtable uniqueNames, XmlSchemaElement elem, string constraintPrefix, DataRelationCollection relations)
+		private void AddForeignKeys (Hashtable uniqueNames, XmlSchemaElement elem, string constraintPrefix, DataRelationCollection relations, XmlDocument doc)
 		{
 			if (relations == null) return;
 			
-			XmlDocument doc = new XmlDocument();
 			foreach (DataRelation rel in relations) {
 				
 				if (rel.ParentKeyConstraint == null || rel.ChildKeyConstraint == null)
@@ -1582,10 +1623,14 @@ namespace System.Data {
 				XmlSchemaXPath field;
 				foreach (DataColumn column in rel.ChildColumns)	{
 				 	field = new XmlSchemaXPath();
-					field.XPath = constraintPrefix+column.ColumnName;
+					string typePrefix = column.ColumnMapping == MappingType.Attribute ? "@" : "";
+					field.XPath = typePrefix + constraintPrefix + column.ColumnName;
 					keyRef.Fields.Add(field);
 				}
+
 				keyRef.UnhandledAttributes = (XmlAttribute[])attrs.ToArray (typeof (XmlAttribute));
+				AddExtendedPropertyAttributes (keyRef, rel.ExtendedProperties, doc);
+
 				elem.Constraints.Add (keyRef);
 			}
 		}
@@ -1596,6 +1641,9 @@ namespace System.Data {
 			ArrayList atts;
 			DataColumn simple;
 			
+			ArrayList xattrs = new ArrayList();
+			XmlAttribute xattr;
+
 			SplitColumns (table, out atts, out elements, out simple);
 
 			XmlSchemaElement elem = new XmlSchemaElement ();
@@ -1637,8 +1685,6 @@ namespace System.Data {
 					
 					// Add element for the column.
 					XmlSchemaElement colElem = new XmlSchemaElement ();
-					ArrayList xattrs = new ArrayList();
-					XmlAttribute xattr;
 					colElem.Name = col.ColumnName;
 				
 					if (col.ColumnName != col.Caption && col.Caption != String.Empty) {
@@ -1661,7 +1707,13 @@ namespace System.Data {
 
 					if (col.DefaultValue.ToString () != String.Empty)
 						colElem.DefaultValue = WriteObjectXml (col.DefaultValue);
-					
+
+					if (col.ReadOnly) {
+						xattr = doc.CreateAttribute (XmlConstants.MsdataPrefix, XmlConstants.ReadOnly, XmlConstants.MsdataNamespace);
+						xattr.Value = "true";
+						xattrs.Add (xattr);
+					}
+
 					if (col.MaxLength < 0)
 						colElem.SchemaTypeName = MapType (col.DataType);
 					
@@ -1687,6 +1739,8 @@ namespace System.Data {
 					}
 					
 					colElem.UnhandledAttributes = (XmlAttribute[])xattrs.ToArray(typeof (XmlAttribute));
+					xattrs.Clear ();
+					AddExtendedPropertyAttributes (colElem, col.ExtendedProperties, doc);
 					seq.Items.Add (colElem);
 				}
 
@@ -1725,13 +1779,58 @@ namespace System.Data {
 					// FIXME: Handle prefix mapping correctly.
 					schemaToAdd.Namespaces.Add (prefix, col.Namespace);
 				}
-				att.SchemaTypeName = MapType (col.DataType);
+				if (!col.AllowDBNull)
+					att.Use = XmlSchemaUse.Required;
+
+				if (col.MaxLength > -1)
+					att.SchemaType = GetTableSimpleType (doc, col);
+				else
+					att.SchemaTypeName = MapType (col.DataType);
+				// FIXME: what happens if extended properties are set on attribute columns??
+				if (!col.AllowDBNull)
+					att.Use = XmlSchemaUse.Required;
+				if (col.DefaultValue.ToString () != String.Empty)
+					att.DefaultValue = WriteObjectXml (col.DefaultValue);
+
+				if (col.ReadOnly) {
+					xattr = doc.CreateAttribute (XmlConstants.MsdataPrefix, XmlConstants.ReadOnly, XmlConstants.MsdataNamespace);
+					xattr.Value = "true";
+					xattrs.Add (xattr);
+				}
+
+				att.UnhandledAttributes = xattrs.ToArray (typeof (XmlAttribute)) as XmlAttribute [];
+				xattrs.Clear ();
+
+				if (col.MaxLength > -1)
+					att.SchemaType = GetTableSimpleType (doc, col);
+				else
+					att.SchemaTypeName = MapType (col.DataType);
 				schemaAttributes.Add (att);
 			}
 
+			AddExtendedPropertyAttributes (elem, table.ExtendedProperties, doc);
+
 			return elem;
 		}
 
+		private void AddExtendedPropertyAttributes (XmlSchemaAnnotated xsobj, PropertyCollection props, XmlDocument doc)
+		{
+			ArrayList attList = new ArrayList ();
+			XmlAttribute xmlAttr;
+
+			if (xsobj.UnhandledAttributes != null)
+				attList.AddRange (xsobj.UnhandledAttributes);
+
+			// add extended properties to xs:element
+			foreach (DictionaryEntry de in props) {
+				xmlAttr = doc.CreateAttribute (XmlConstants.MspropPrefix, XmlConvert.EncodeName (de.Key.ToString ()), XmlConstants.MspropNamespace);
+				xmlAttr.Value = de.Value != null ? WriteObjectXml (de.Value) : String.Empty;
+				attList.Add (xmlAttr);
+			}
+			if (attList.Count > 0)
+				xsobj.UnhandledAttributes = attList.ToArray (typeof (XmlAttribute)) as XmlAttribute [];
+ 		}
+
 		private string SafeNS (string ns)
 		{
 			return ns != null ? ns : String.Empty;
