@@ -176,14 +176,8 @@ namespace System.Xml
 		}
 #endif
 
-		public override bool EOF
-		{
-			get
-			{
-				return
-					readState == ReadState.EndOfFile ||
-					readState == ReadState.Closed;
-			}
+		public override bool EOF {
+			get { return readState == ReadState.EndOfFile; }
 		}
 
 #if NET_2_0
@@ -688,7 +682,7 @@ namespace System.Xml
 
 		public void ResetState ()
 		{
-			Init ();
+			throw new InvalidOperationException ("Cannot call ResetState when parsing an XML fragment.");
 		}
 
 		public override void ResolveEntity ()
@@ -1304,6 +1298,9 @@ namespace System.Xml
 					"Multiple document element was detected.");
 			currentState = XmlNodeType.Element;
 
+			currentLinkedNodeLineNumber = line;
+			currentLinkedNodeLinePosition = column;
+
 			parserContext.NamespaceManager.PushScope ();
 
 			string name = ReadName ();
@@ -1406,6 +1403,9 @@ namespace System.Xml
 				throw new XmlException (this as IXmlLineInfo,
 					"End tag cannot appear in this state.");
 
+			currentLinkedNodeLineNumber = line;
+			currentLinkedNodeLinePosition = column;
+
 			string name = ReadName ();
 			if (elementNameStackPos == 0)
 				throw new XmlException (this as IXmlLineInfo,"closing element without matching opening element");
@@ -1548,9 +1548,15 @@ namespace System.Xml
 					ch = ReadReference (false);
 					if (returnEntityReference) // Returns -1 if char validation should not be done
 						break;
-				}
-				else {
-					if (XmlChar.IsInvalid (ch))
+				} else if (normalization && ch == '\r') {
+					ReadChar ();
+					ch = ReadChar ();
+					if (ch != '\n')
+						// append '\n' instead of '\r'.
+						AppendValueChar ('\n');
+					// and in case of "\r\n", discard '\r'.
+				} else {
+					if (normalization && XmlChar.IsInvalid (ch))
 						throw new XmlException (this, "Not allowed character was found.");
 					ch = ReadChar ();
 				}
@@ -2142,8 +2148,18 @@ namespace System.Xml
 						skip = true;
 					}
 				}
-				if (normalization && XmlChar.IsInvalid (ch))
-					throw new XmlException (this, "Invalid character was found.");
+				if (normalization) {
+					if (ch == '\r') {
+						ch = PeekChar ();
+						if (ch != '\n')
+							// append '\n' instead of '\r'.
+							AppendValueChar ('\n');
+						// otherwise, discard '\r'.
+						continue;
+					}
+					else if (XmlChar.IsInvalid (ch))
+						throw new XmlException (this, "Invalid character was found.");
+				}
 
 				AppendValueChar (ch);
 			}
@@ -2333,10 +2349,14 @@ namespace System.Xml
 				case -1:
 					throw new XmlException (this as IXmlLineInfo,"unexpected end of file at DTD.");
 				case '<':
-					if (State == DtdInputState.InsideDoubleQuoted ||
-						State == DtdInputState.InsideSingleQuoted)
+					switch (State) {
+					case DtdInputState.InsideDoubleQuoted:
+					case DtdInputState.InsideSingleQuoted:
+					case DtdInputState.Comment:
 						continue;	// well-formed
-					switch (ReadChar ()) {
+					}
+					int c = ReadChar ();
+					switch (c) {
 					case '?':
 						stateStack.Push (DtdInputState.PI);
 						break;
@@ -2371,7 +2391,7 @@ namespace System.Xml
 						}
 						break;
 					default:
-						throw new XmlException (this as IXmlLineInfo,"unexpected '>'.");
+						throw new XmlException (this as IXmlLineInfo, String.Format ("unexpected '<{0}'.", (char) c));
 					}
 					break;
 				case '\'':
@@ -2398,9 +2418,7 @@ namespace System.Xml
 						stateStack.Pop ();
 						break;
 					case DtdInputState.InsideDoubleQuoted:
-						continue;
 					case DtdInputState.InsideSingleQuoted:
-						continue; // well-formed
 					case DtdInputState.Comment:
 						continue;
 					default:
