@@ -61,7 +61,6 @@ namespace System.Net
 		EventHandler abortHandler;
 		ReadState readState;
 		internal WebConnectionData Data;
-		WebConnectionStream prevStream;
 		bool chunkedRead;
 		ChunkStream chunkStream;
 		AutoResetEvent goAhead;
@@ -89,12 +88,22 @@ namespace System.Net
 			queue = group.Queue;
 		}
 
-		public void Connect ()
+		bool CanReuse ()
+		{
+			// The real condition is !(socket.Poll (0, SelectMode.SelectRead) || socket.Available != 0)
+			// but if there's data pending to read (!) we won't reuse the socket.
+			return (socket.Poll (0, SelectMode.SelectRead) == false);
+		}
+		
+		void Connect ()
 		{
 			lock (this) {
 				if (socket != null && socket.Connected && status == WebExceptionStatus.Success) {
-					reused = true;
-					return;
+					// Take the chunked stream to the expected state (State.None)
+					if (CanReuse () && CompleteChunkedRead ()) {
+						reused = true;
+						return;
+					}
 				}
 
 				reused = false;
@@ -102,7 +111,7 @@ namespace System.Net
 					socket.Close();
 					socket = null;
 				}
-				
+
 				chunkStream = null;
 				IPHostEntry hostEntry = sPoint.HostEntry;
 
@@ -148,17 +157,94 @@ namespace System.Net
 			}
 		}
 
+		bool CreateTunnel (HttpWebRequest request, Stream stream, out byte [] buffer)
+		{
+			StringBuilder sb = new StringBuilder ();
+			sb.Append ("CONNECT ");
+			sb.Append (request.Address.Host);
+			sb.Append (':');
+			sb.Append (request.Address.Port);
+			sb.Append (" HTTP/");
+			if (request.ServicePoint.ProtocolVersion == HttpVersion.Version11)
+				sb.Append ("1.1");
+			else
+				sb.Append ("1.0");
+
+			sb.Append ("\r\nHost: ");
+			sb.Append (request.Address.Authority);
+			if (request.Headers ["Proxy-Authorization"] != null) {
+				sb.Append ("\r\nProxy-Authorization: ");
+				sb.Append (request.Headers ["Proxy-Authorization"]);
+			}
+				
+			sb.Append ("\r\n\r\n");
+			byte [] connectBytes = Encoding.Default.GetBytes (sb.ToString ());
+			stream.Write (connectBytes, 0, connectBytes.Length);
+			return ReadHeaders (request, stream, out buffer);
+		}
+
+		bool ReadHeaders (HttpWebRequest request, Stream stream, out byte [] retBuffer)
+		{
+			retBuffer = null;
+
+			byte [] buffer = new byte [256];
+			MemoryStream ms = new MemoryStream ();
+			bool gotStatus = false;
+
+			while (true) {
+				int n = stream.Read (buffer, 0, 256);
+				if (n == 0) {
+					HandleError (WebExceptionStatus.ServerProtocolViolation, null, "ReadHeders");
+					return false;
+				}
+				
+				ms.Write (buffer, 0, n);
+				int start = 0;
+				string str = null;
+				while (ReadLine (ms.GetBuffer (), ref start, (int) ms.Length, ref str)) {
+					if (str == null) {
+						if (ms.Length - start > 0) {
+							retBuffer = new byte [ms.Length - start];
+							Buffer.BlockCopy (ms.GetBuffer (), start, retBuffer, 0, retBuffer.Length);
+						}
+						return true;
+					}
+
+					if (gotStatus)
+						continue;
+
+					int spaceidx = str.IndexOf (' ');
+					if (spaceidx == -1)
+						throw new Exception ();
+
+					int resultCode = Int32.Parse (str.Substring (spaceidx + 1, 3));
+					if (resultCode != 200)
+						throw new Exception ();
+
+					gotStatus = true;
+				}
+			}
+		}
+
 		bool CreateStream (HttpWebRequest request)
 		{
 			try {
 				NetworkStream serverStream = new NetworkStream (socket, false);
+
 				if (request.Address.Scheme == Uri.UriSchemeHttps) {
 					ssl = true;
 					EnsureSSLStreamAvailable ();
 					if (!reused || nstream == null || nstream.GetType () != sslStream) {
-						object[] args = new object [3] { serverStream,
+						byte [] buffer = null;
+						if (sPoint.UseConnect) {
+							bool ok = CreateTunnel (request, serverStream, out buffer);
+							if (!ok)
+								return false;
+						}
+
+						object[] args = new object [4] { serverStream,
 										request.ClientCertificates,
-										request };
+										request, buffer};
 						nstream = (Stream) Activator.CreateInstance (sslStream, args);
 					}
 					// we also need to set ServicePoint.Certificate 
@@ -177,15 +263,12 @@ namespace System.Net
 			return true;
 		}
 		
-		void HandleError (WebExceptionStatus st, Exception e)
+		void HandleError (WebExceptionStatus st, Exception e, string where)
 		{
 			status = st;
 			lock (this) {
-				busy = false;
 				if (st == WebExceptionStatus.RequestCanceled)
 					Data = new WebConnectionData ();
-
-				status = st;
 			}
 
 			if (e == null) { // At least we now where it comes from
@@ -197,7 +280,7 @@ namespace System.Net
 			}
 
 			if (Data != null && Data.request != null)
-				Data.request.SetResponseError (st, e);
+				Data.request.SetResponseError (st, e, where);
 
 			Close (true);
 		}
@@ -216,23 +299,20 @@ namespace System.Net
 			try {
 				nread = ns.EndRead (result);
 			} catch (Exception e) {
-				cnc.status = WebExceptionStatus.ReceiveFailure;
-				cnc.HandleError (cnc.status, e);
+				cnc.HandleError (WebExceptionStatus.ReceiveFailure, e, "ReadDone1");
 				return;
 			}
 
 			if (nread == 0) {
-				cnc.status = WebExceptionStatus.ReceiveFailure;
-				cnc.HandleError (cnc.status, null);
+				cnc.HandleError (WebExceptionStatus.ReceiveFailure, null, "ReadDone2");
 				return;
 			}
 
 			if (nread < 0) {
-				cnc.HandleError (WebExceptionStatus.ServerProtocolViolation, null);
+				cnc.HandleError (WebExceptionStatus.ServerProtocolViolation, null, "ReadDone3");
 				return;
 			}
 
-			//Console.WriteLine (System.Text.Encoding.Default.GetString (cnc.buffer, 0, nread + cnc.position));
 			int pos = -1;
 			nread += cnc.position;
 			if (cnc.readState == ReadState.None) { 
@@ -244,7 +324,7 @@ namespace System.Net
 				}
 
 				if (exc != null) {
-					cnc.HandleError (WebExceptionStatus.ServerProtocolViolation, exc);
+					cnc.HandleError (WebExceptionStatus.ServerProtocolViolation, exc, "ReadDone4");
 					return;
 				}
 			}
@@ -272,10 +352,20 @@ namespace System.Net
 				stream.ReadBufferOffset = pos;
 				stream.ReadBufferSize = nread;
 			} else if (cnc.chunkStream == null) {
-				cnc.chunkStream = new ChunkStream (cnc.buffer, pos, nread, data.Headers);
+				try {
+					cnc.chunkStream = new ChunkStream (cnc.buffer, pos, nread, data.Headers);
+				} catch (Exception e) {
+					cnc.HandleError (WebExceptionStatus.ServerProtocolViolation, e, "ReadDone5");
+					return;
+				}
 			} else {
 				cnc.chunkStream.ResetBuffer ();
-				cnc.chunkStream.Write (cnc.buffer, pos, nread);
+				try {
+					cnc.chunkStream.Write (cnc.buffer, pos, nread);
+				} catch (Exception e) {
+					cnc.HandleError (WebExceptionStatus.ServerProtocolViolation, e, "ReadDone6");
+					return;
+				}
 			}
 
 			data.stream = stream;
@@ -288,7 +378,6 @@ namespace System.Net
 					if (cnc.queue.Count > 0) {
 						stream.ReadAll ();
 					} else {
-						cnc.prevStream = stream;
 						stream.CheckComplete ();
 					}
 				}
@@ -320,7 +409,7 @@ namespace System.Net
 				int size = cnc.buffer.Length - cnc.position;
 				ns.BeginRead (cnc.buffer, cnc.position, size, readDoneDelegate, cnc);
 			} catch (Exception e) {
-				cnc.HandleError (WebExceptionStatus.ReceiveFailure, e);
+				cnc.HandleError (WebExceptionStatus.ReceiveFailure, e, "InitRead");
 			}
 		}
 		
@@ -330,12 +419,19 @@ namespace System.Net
 			string line = null;
 			bool lineok = false;
 			bool isContinue = false;
+			bool emptyFirstLine = false;
 			do {
 				if (readState == ReadState.None) {
 					lineok = ReadLine (buffer, ref pos, max, ref line);
 					if (!lineok)
 						return -1;
 
+					if (line == null) {
+						emptyFirstLine = true;
+						continue;
+					}
+					emptyFirstLine = false;
+
 					readState = ReadState.Status;
 
 					string [] parts = line.Split (' ');
@@ -360,6 +456,7 @@ namespace System.Net
 						return pos;
 				}
 
+				emptyFirstLine = false;
 				if (readState == ReadState.Status) {
 					readState = ReadState.Headers;
 					Data.Headers = new WebHeaderCollection ();
@@ -413,7 +510,7 @@ namespace System.Net
 						return pos;
 					}
 				}
-			} while (isContinue == true);
+			} while (emptyFirstLine || isContinue);
 
 			return -1;
 		}
@@ -433,7 +530,6 @@ namespace System.Net
 			keepAlive = request.KeepAlive;
 			Data = new WebConnectionData ();
 			Data.request = request;
-
 			Connect ();
 			if (status != WebExceptionStatus.Success) {
 				request.SetWriteStreamError (status);
@@ -455,11 +551,6 @@ namespace System.Net
 		internal EventHandler SendRequest (HttpWebRequest request)
 		{
 			lock (this) {
-				if (prevStream != null && socket != null && socket.Connected) {
-					prevStream.ReadAll ();
-					prevStream = null;
-				}
-
 				if (!busy) {
 					busy = true;
 					ThreadPool.RegisterWaitForSingleObject (goAhead, initConn,
@@ -478,7 +569,6 @@ namespace System.Net
 		{
 			lock (queue) {
 				if (queue.Count > 0) {
-					prevStream = null;
 					SendRequest ((HttpWebRequest) queue.Dequeue ());
 				}
 			}
@@ -490,7 +580,7 @@ namespace System.Net
 				busy = false;
 				string header = (sPoint.UsesProxy) ? "Proxy-Connection" : "Connection";
 				string cncHeader = (Data.Headers != null) ? Data.Headers [header] : null;
-				bool keepAlive = (Data.Version == HttpVersion.Version11);
+				bool keepAlive = (Data.Version == HttpVersion.Version11 && this.keepAlive);
 				if (cncHeader != null) {
 					cncHeader = cncHeader.ToLower ();
 					keepAlive = (this.keepAlive && cncHeader.IndexOf ("keep-alive") != -1);
@@ -504,7 +594,6 @@ namespace System.Net
 				goAhead.Set ();
 				lock (queue) {
 					if (queue.Count > 0) {
-						prevStream = null;
 						SendRequest ((HttpWebRequest) queue.Dequeue ());
 					}
 				}
@@ -553,6 +642,7 @@ namespace System.Net
 			return true;
 		}
 
+
 		internal IAsyncResult BeginRead (byte [] buffer, int offset, int size, AsyncCallback cb, object state)
 		{
 			if (nstream == null)
@@ -563,7 +653,7 @@ namespace System.Net
 				try {
 					result = nstream.BeginRead (buffer, offset, size, cb, state);
 				} catch (Exception) {
-					status = WebExceptionStatus.ReceiveFailure;
+					HandleError (WebExceptionStatus.ReceiveFailure, null, "chunked BeginRead");
 					throw;
 				}
 			}
@@ -588,26 +678,71 @@ namespace System.Net
 				if (wr.InnerAsyncResult != null)
 					nbytes = nstream.EndRead (wr.InnerAsyncResult);
 
-				chunkStream.WriteAndReadBack (wr.Buffer, wr.Offset, wr.Size, ref nbytes);
-				if (nbytes < wr.Size && chunkStream.WantMore) {
-					int size = chunkStream.ChunkLeft;
-					if (size < 0) // not read chunk size yet
-						size = 1024;
+				bool done = (nbytes == 0);
+				try {
+					chunkStream.WriteAndReadBack (wr.Buffer, wr.Offset, wr.Size, ref nbytes);
+					if (!done && nbytes == 0 && chunkStream.WantMore)
+						nbytes = EnsureRead (wr.Buffer, wr.Offset, wr.Size);
+				} catch (Exception e) {
+					if (e is WebException)
+						throw e;
 
-					byte [] morebytes = new byte [size];
-					int nread;
-					nread = nstream.Read (morebytes, 0, size);
-					chunkStream.Write (morebytes, 0, nread);
-					morebytes = null;
-					nbytes += chunkStream.Read (wr.Buffer, wr.Offset + nbytes,
-								wr.Size - nbytes);
+					throw new WebException ("Invalid chunked data.", e,
+								WebExceptionStatus.ServerProtocolViolation, null);
 				}
+
+				if ((done || nbytes == 0) && chunkStream.WantMore) {
+					HandleError (WebExceptionStatus.ReceiveFailure, null, "chunked EndRead");
+					throw new WebException ("Read error", null, WebExceptionStatus.ReceiveFailure, null);
+				}
+
 				return nbytes;
 			}
 
 			return nstream.EndRead (result);
 		}
 
+		// To be called on chunkedRead when we can read no data from the ChunkStream yet
+		int EnsureRead (byte [] buffer, int offset, int size)
+		{
+			byte [] morebytes = null;
+			int nbytes = 0;
+			while (nbytes == 0 && chunkStream.WantMore) {
+				int localsize = chunkStream.ChunkLeft;
+				if (localsize <= 0) // not read chunk size yet
+					localsize = 1024;
+				else if (localsize > 16384)
+					localsize = 16384;
+
+				if (morebytes == null || morebytes.Length < localsize)
+					morebytes = new byte [localsize];
+
+				int nread = nstream.Read (morebytes, 0, localsize);
+				if (nread <= 0)
+					return 0; // Error
+
+				chunkStream.Write (morebytes, 0, nread);
+				nbytes += chunkStream.Read (buffer, offset + nbytes, size - nbytes);
+			}
+
+			return nbytes;
+		}
+
+		bool CompleteChunkedRead()
+		{
+			if (!chunkedRead || chunkStream == null)
+				return true;
+
+			while (chunkStream.WantMore) {
+				int nbytes = nstream.Read (buffer, 0, buffer.Length);
+				if (nbytes <= 0)
+					return false; // Socket was disconnected
+
+				chunkStream.Write (buffer, 0, nbytes);
+			}
+
+			return true;
+  		}
 		internal IAsyncResult BeginWrite (byte [] buffer, int offset, int size, AsyncCallback cb, object state)
 		{
 			IAsyncResult result = null;
@@ -637,14 +772,29 @@ namespace System.Net
 
 			int result = 0;
 			try {
-				if (!chunkedRead || chunkStream.WantMore)
+				bool done = false;
+				if (!chunkedRead) {
 					result = nstream.Read (buffer, offset, size);
+					done = (result == 0);
+				}
 
-				if (chunkedRead)
-					chunkStream.WriteAndReadBack (buffer, offset, size, ref result);
+				if (chunkedRead) {
+					try {
+						chunkStream.WriteAndReadBack (buffer, offset, size, ref result);
+						if (!done && result == 0 && chunkStream.WantMore)
+							result = EnsureRead (buffer, offset, size);
+					} catch (Exception e) {
+						HandleError (WebExceptionStatus.ReceiveFailure, e, "chunked Read1");
+						throw;
+					}
+
+					if ((done || result == 0) && chunkStream.WantMore) {
+						HandleError (WebExceptionStatus.ReceiveFailure, null, "chunked Read2");
+						throw new WebException ("Read error", null, WebExceptionStatus.ReceiveFailure, null);
+					}
+				}
 			} catch (Exception e) {
-				status = WebExceptionStatus.ReceiveFailure;
-				HandleError (status, e);
+				HandleError (WebExceptionStatus.ReceiveFailure, e, "Read");
 			}
 
 			return result;
@@ -669,7 +819,7 @@ namespace System.Net
 		{
 			lock (this) {
 				if (!reused) {
-					HandleError (WebExceptionStatus.SendFailure, null);
+					HandleError (WebExceptionStatus.SendFailure, null, "TryReconnect");
 					return false;
 				}
 
@@ -677,12 +827,12 @@ namespace System.Net
 				reused = false;
 				Connect ();
 				if (status != WebExceptionStatus.Success) {
-					HandleError (WebExceptionStatus.SendFailure, null);
+					HandleError (WebExceptionStatus.SendFailure, null, "TryReconnect2");
 					return false;
 				}
 			
 				if (!CreateStream (Data.request)) {
-					HandleError (WebExceptionStatus.SendFailure, null);
+					HandleError (WebExceptionStatus.SendFailure, null, "TryReconnect3");
 					return false;
 				}
 			}
@@ -707,16 +857,15 @@ namespace System.Net
 					socket = null;
 				}
 
-				if (sendNext) {
-					goAhead.Set ();
+				goAhead.Set ();
+				if (sendNext)
 					SendNext ();
-				}
 			}
 		}
 
 		void Abort (object sender, EventArgs args)
 		{
-			HandleError (WebExceptionStatus.RequestCanceled, null);
+			HandleError (WebExceptionStatus.RequestCanceled, null, "Abort");
 		}
 
 		internal bool Busy {
@@ -730,11 +879,6 @@ namespace System.Net
 				}
 			}
 		}
-		
-		~WebConnection ()
-		{
-			Close (false);
-		}
 	}
 }
 
