@@ -252,6 +252,8 @@ namespace System.Reflection.Emit {
 		}
 
 		private TypeBuilder DefineType (string name, TypeAttributes attr, Type parent, Type[] interfaces, PackingSize packsize, int typesize) {
+			if (name_cache.Contains (name))
+				throw new ArgumentException ("Duplicate type name within an assembly.");
 			TypeBuilder res = new TypeBuilder (this, name, attr, parent, interfaces, packsize, typesize, null);
 			if (types != null) {
 				if (types.Length == num_types) {
@@ -433,7 +435,7 @@ namespace System.Reflection.Emit {
 			if (types == null)
 				return new TypeBuilder [0];
 
-			int n = types.Length;
+			int n = num_types;
 			TypeBuilder [] copy = new TypeBuilder [n];
 			Array.Copy (types, copy, n);
 
@@ -624,9 +626,15 @@ namespace System.Reflection.Emit {
 
 		internal void Save ()
 		{
-			if (transient)
+			if (transient && !is_main)
 				return;
 
+			if (types != null) {
+				for (int i = 0; i < num_types; ++i)
+					if (!types [i].is_created)
+						throw new NotSupportedException ("Type '" + types [i].FullName + "' was not completed.");
+			}
+
 			if ((global_type != null) && (global_type_created == null))
 				global_type_created = global_type.CreateType ();
 
