@@ -1007,8 +1007,10 @@ namespace System
 						num = 1;
 					}
 
-					if (hour >= 12)
+					if (hour > 12)
 						return false;
+					if (hour == 12)
+						hour = 0;
 
 					break;
 				case 'H':
@@ -1188,6 +1190,22 @@ namespace System
 
 				s = s.Substring (num_parsed);
 
+				if (!exact) {
+					switch (chars [pos]) {
+					case 'm':
+					case 's':
+					case 'f':
+					case 'z':
+						if (s.Length > 0 && s [0] == 'Z'
+						    && (pos + 1 == chars.Length
+						    || chars [pos + 1] != 'Z')) {
+							useutc = true;
+							s = s.Substring (1);
+						}
+						break;
+					}
+				}
+
 				pos = pos + num + 1;
 				num = 0;
 			}
