@@ -1654,7 +1654,7 @@ namespace Mono.CSharp
 							margs [1] = spec.Substring (0, cp);
 						} else {
 							margs [1] = spec;
-							margs [0] = spec.Replace ('/','.').Replace ('\\', '.');
+							margs [0] = Path.GetFileName (spec);
 						}
 
 						if (File.Exists ((string) margs [1]))
